require "util"

function calc_power(mk, model)
local power = 0

--For mk1: value in table = (10^(X/50))*44000
--For mk2: value in table = (10^(X/49,5))*50000
--For mk3: value in table = (10^(X/49))*56000
--For mk4: value in table = (10^(X/48,5))*62000
--Remember that above values are in Watts, you have to recalculate it to energy_per_movement/rotation using formula bellow:
--energy_per_movement = (100*power/315)/2 -- conversion from Power in W to energy_per_movement or rotation in J
--energy_per_movement = math.floor(power+1) -- with this line consumption is little higher... exactly like in my excel


model = (model+1)/2  -- conversion from 1-256 to 1-128,5     for 256 it is (256+1)/2=257/2=128,5     for 1 it is (1+1)/2=2/2=1
if model < 1 then model = 0 end -- conversion... only for 0.... (0+1)/2=0,5 but should be 0

-- bellow... base is purposely duplicate because you can change it independent for mk1,2,3,4...
if mk == 1 then
	local base = 10
	local exponent = (model/50)
	power = math.pow(base,exponent)
	power = power * 44000
elseif mk == 2 then
	local base = 10
	local exponent = (model/49.5)
	power = math.pow(base,exponent)
	power = power * 50000
elseif mk == 3 then
	local base = 10
	local exponent = (model/49)
	power = math.pow(base,exponent)
	power = power * 56000
elseif mk == 4 then
	local base = 10
	local exponent = (model/48.5)
	power = math.pow(base,exponent)
	power = power * 62000
end

power = (100*power/315)/2 -- conversion from Power in W to energy_per_movement or rotation in J
power = math.floor(power+1) -- with this line consumption is little higher... exactly like in my excel

return power
end

local constants = {
  type="inserter",
  name="tele-inserter-mk",
  icon="__Telelogistics__/graphics/icons/",
  flags={"placeable-neutral", "placeable-player", "player-creation"}, 
  minable={hardness = 0.2, mining_time = 0.5, result = "tele-inserter-mk" --[[ name?--]]},
  max_health=40,
  corpse = "small-remnants",
  resistances={{type = "fire", percent = 90}},
  collision_box = {{-0.15, -0.15}, {0.15, 0.15}},
  selection_box={{-0.4, -0.35}, {0.4, 0.45}},
  pickup_position = {0, -1},
  insert_position = {0, 1.35},
  energy_per_movement=6985,
  energy_per_rotation=6985,
  energy_source = {type = "electric", usage_priority = "secondary-input",
  drain = {value=5, ext="kW"}},
  extension_speed = 0.07, fast_replaceable_group = "inserter",
  hand_base_picture = {
    filename = "__Telelogistics__/graphics/entity/tele-inserter/tele-inserter-hand-base.png",
    priority = "extra-high",
    width = 8,
    height = 34
  },
  hand_closed_picture = {
    filename = "__Telelogistics__/graphics/entity/tele-inserter/tele-inserter-hand-closed.png",
    priority = "extra-high",
    width = 20,
    height = 41
  },
  hand_open_picture = {
    filename = "__Telelogistics__/graphics/entity/tele-inserter/tele-inserter-hand-open.png",
    priority = "extra-high",
    width = 20,
    height = 41
  },
  platform_picture =
  {
    priority = "extra-high",
    width = 46,
    height = 46,
    sheet = "__Telelogistics__/graphics/entity/tele-inserter/tele-inserter-platform-mk"
  },
  rotation_speed = 0.035  --0.035
  }

local max_number_of_entities = 256 -- you can read this line as MAX teleinserter distance....change only here no further adjustments needed
  
for h = 1, 4 do -- loop from 1 to 4
  for i = 0, max_number_of_entities do -- mkH to mkH-256
    nextEntity = util.table.deepcopy(constants) -- just making a copy
    local modularName = nextEntity.name .. h
	nextEntity.icon = nextEntity.icon .. modularName .. ".png"
	if i ~= 0 then
      nextEntity.order = modularName .. "-place"
      nextEntity.name = modularName .. "-" .. i
    else
      nextEntity.name = modularName
    end
    --nextEntity.icon = nextEntity.icon .. modularName .. ".png"  -- moved up
    nextEntity.minable.result = modularName
    nextEntity.energy_source.drain = (nextEntity.energy_source.drain.value * h) .. nextEntity.energy_source.drain.ext
    --nextEntity.energy_per_movement = power["tele_power_mk" .. h][i+1]
    nextEntity.energy_per_movement = calc_power(h, i)
    nextEntity.energy_per_rotation = nextEntity.energy_per_movement
	nextEntity.platform_picture.sheet = nextEntity.platform_picture.sheet .. h .. ".png"
    data:extend({nextEntity})
  end
end

data:extend({ -- and don't forget the chests...
{
    type = "container",
    name = "tele-chest-mk1",
    icon = "__Telelogistics__/graphics/icons/tele-chest-mk1.png",
    flags = {"placeable-neutral", "player-creation"},
    minable = {mining_time = 1, result = "tele-chest-mk1"},
    max_health = 100,
    corpse = "small-remnants",
    open_sound = { filename = "__base__/sound/metallic-chest-open.ogg", volume=0.65 },
    close_sound = { filename = "__base__/sound/metallic-chest-close.ogg", volume = 0.7 },
    resistances =
    {
      {
        type = "fire",
        percent = 80
      }
    },
    collision_box = {{-0.35, -0.35}, {0.35, 0.35}},
    selection_box = {{-0.5, -0.5}, {0.5, 0.5}},
    --fast_replaceable_group = "container",
    inventory_size = 1,
    picture =
    {
      filename = "__Telelogistics__/graphics/entity/tele-chest/tele-chest-mk1.png",
      priority = "extra-high",
      width = 48,
      height = 34,
      shift = {0.2, 0}
    }
    },
    {
    type = "container",
    name = "tele-chest-mk2",
    icon = "__Telelogistics__/graphics/icons/tele-chest-mk2.png",
    flags = {"placeable-neutral", "player-creation"},
    minable = {mining_time = 1, result = "tele-chest-mk2"},
    max_health = 100,
    corpse = "small-remnants",
    open_sound = { filename = "__base__/sound/metallic-chest-open.ogg", volume=0.65 },
    close_sound = { filename = "__base__/sound/metallic-chest-close.ogg", volume = 0.7 },
    resistances =
    {
      {
        type = "fire",
        percent = 80
      }
    },
    collision_box = {{-0.35, -0.35}, {0.35, 0.35}},
    selection_box = {{-0.5, -0.5}, {0.5, 0.5}},
    --fast_replaceable_group = "container",
    inventory_size = 8,
    picture =
    {
      filename = "__Telelogistics__/graphics/entity/tele-chest/tele-chest-mk2.png",
      priority = "extra-high",
      width = 48,
      height = 34,
      shift = {0.2, 0}
    }
  },
  {
    type = "container",
    name = "tele-chest-mk3",
    icon = "__Telelogistics__/graphics/icons/tele-chest-mk3.png",
    flags = {"placeable-neutral", "player-creation"},
    minable = {mining_time = 1, result = "tele-chest-mk3"},
    max_health = 100,
    corpse = "small-remnants",
    open_sound = { filename = "__base__/sound/metallic-chest-open.ogg", volume=0.65 },
    close_sound = { filename = "__base__/sound/metallic-chest-close.ogg", volume = 0.7 },
    resistances =
    {
      {
        type = "fire",
        percent = 80
      }
    },
    collision_box = {{-0.35, -0.35}, {0.35, 0.35}},
    selection_box = {{-0.5, -0.5}, {0.5, 0.5}},
    --fast_replaceable_group = "container",
    inventory_size = 16,
    picture =
    {
      filename = "__Telelogistics__/graphics/entity/tele-chest/tele-chest-mk3.png",
      priority = "extra-high",
      width = 48,
      height = 34,
      shift = {0.2, 0}
    }
  },
  
   {
    type = "lamp",
    name = "teleporter",
    icon = "__Telelogistics__/graphics/icons/teleport.png",
    flags = {"placeable-neutral", "player-creation"},
    minable = {hardness = 0.2, mining_time = 0.5, result = "teleporter"},
    max_health = 55,
    corpse = "small-remnants",
    --collision_box = {{-0, -0}, {0, 0}},
    selection_box = {{-1.75, -0.3}, {1.75, 1.55}},
	order = "tteleport",
    energy_source =
    {
      type = "electric",
      usage_priority = "secondary-input",
	  drain = "120kW"
    },
    energy_usage_per_tick = "120kW",
    light = {intensity = 0, size = 0},
    picture_off =
    {
      filename = "__Telelogistics__/graphics/entity/teleport/inactive.png",
      priority = "high",
      width = 100,
      height = 106,
      shift = {0, 0}
    },
    picture_on =
    {
      filename = "__Telelogistics__/graphics/entity/teleport/inactive.png",
      priority = "high",
      width = 100,
      height = 106,
      shift = {0, 0}
    }
  } ,
    {
    type = "lamp",
    name = "teleporter-active",
    icon = "__Telelogistics__/graphics/icons/teleport.png",
    flags = {"placeable-neutral", "player-creation"},
    minable = {hardness = 0.2, mining_time = 0.5, result = "teleporter"},
    max_health = 55,
    corpse = "small-remnants",
   -- collision_box = {{-0, -0}, {0, 0}},
    selection_box = {{-1.75, -0.3}, {1.75, 1.55}},
	order = "tteleport",
    energy_source =
    {
      type = "electric",
      usage_priority = "secondary-input",
	  drain = "5000kW"
    },
    energy_usage_per_tick = "5000KW",
    light = {intensity = 0.5, size = 3},   --{intensity = 0.9, size = 40},
    picture_off =
    {
      filename = "__Telelogistics__/graphics/entity/teleport/active.png",
      priority = "high",
      width = 100,
      height = 106,
      shift = {0, 0}
    },
    picture_on =
    {
      filename = "__Telelogistics__/graphics/entity/teleport/active.png",
      priority = "high",
      width = 100,
      height = 106,
      shift = {0, 0}
    }
  } ,
   {
    type = "lamp",
    name = "teleporter-green",
    icon = "__Telelogistics__/graphics/icons/teleport.png",
    flags = {"placeable-neutral", "player-creation"},
    minable = {hardness = 0.2, mining_time = 0.5, result = "teleporter"},
    max_health = 55,
    corpse = "small-remnants",
    --collision_box = {{-0, -0}, {0, 0}},
    selection_box = {{-1.75, -0.3}, {1.75, 1.55}},
	order = "tteleport",
    energy_source =
    {
      type = "electric",
      usage_priority = "secondary-input",
	  drain = "120kW"
    },
    energy_usage_per_tick = "120kW",
    light = {intensity = 0.5, size = 3},
    picture_off =
    {
      filename = "__Telelogistics__/graphics/entity/teleport/activegreen.png",
      priority = "high",
      width = 100,
      height = 106,
      shift = {0, 0}
    },
    picture_on =
    {
      filename = "__Telelogistics__/graphics/entity/teleport/activegreen.png",
      priority = "high",
      width = 100,
      height = 106,
      shift = {0, 0}
    }
  },
      {
    type = "lamp",
    name = "teleporter-activenp",
    icon = "__Telelogistics__/graphics/icons/teleport.png",
    flags = {"placeable-neutral", "player-creation"},
    minable = {hardness = 0.2, mining_time = 0.5, result = "teleporter"},
    max_health = 55,
    corpse = "small-remnants",
   -- collision_box = {{-0, -0}, {0, 0}},
    selection_box = {{-1.75, -0.3}, {1.75, 1.55}},
	order = "tteleport",
    energy_source =
    {
      type = "electric",
      usage_priority = "secondary-input",
	  drain = "120kW"
    },
    energy_usage_per_tick = "120kW",
    light = {intensity = 0.5, size = 3},   --{intensity = 0.9, size = 40},
    picture_off =
    {
      filename = "__Telelogistics__/graphics/entity/teleport/active.png",
      priority = "high",
      width = 100,
      height = 106,
      shift = {0, 0}
    },
    picture_on =
    {
      filename = "__Telelogistics__/graphics/entity/teleport/active.png",
      priority = "high",
      width = 100,
      height = 106,
      shift = {0, 0}
    }
  },
  
 

 
 
   {
    type = "explosion",
    name = "tl_red",
	flags = {"placeable-off-grid"},
    animation_speed = 1,
    animations =
    {
      {
        filename = "__Telelogistics__/graphics/red.png",
        priority = "extra-high",
        frame_width = 2,
        frame_height = 2,
        frame_count = 2
      }
    },
   -- light = {intensity = 0, size = 0},
   -- smoke = "smoke-fast",
  --  smoke_count = 0,
  --  smoke_slow_down_factor = 1,
  },
  
   {
    type = "explosion",
    name = "tl_blue",
	flags = {"not-on-map", "placeable-off-grid"},
    animation_speed = 1,
    animations =
    {
      {
        filename = "__Telelogistics__/graphics/blue.png",
        priority = "extra-high",
        frame_width = 2,
        frame_height = 2,
        frame_count = 2
      }
    },
 --   light = {intensity = 0, size = 0},
 --   smoke = "smoke-fast",
 --   smoke_count = 0,
 --   smoke_slow_down_factor = 1,
  },
 
   {
    type = "explosion",
    name = "tl_green",
	flags = {"not-on-map", "placeable-off-grid"},
    animation_speed = 1,
    animations =
    {
      {
        filename = "__Telelogistics__/graphics/green.png",
        priority = "extra-high",
        frame_width = 2,
        frame_height = 2,
        frame_count = 2
      }
    },
 --   light = {intensity = 0, size = 0},
--    smoke = "smoke-fast",
 --   smoke_count = 0,
 --   smoke_slow_down_factor = 1,
  },
 
   {
    type = "explosion",
    name = "tl_yellow",
	flags = {"placeable-off-grid"},
    animation_speed = 1,
    animations =
    {
      {
        filename = "__Telelogistics__/graphics/yellow.png",
        priority = "extra-high",
        frame_width = 2,
        frame_height = 2,
        frame_count = 2
      }
    },
  --  light = {intensity = 0, size = 0},
  --  smoke = "smoke-fast",
  --  smoke_count = 0,
  --  smoke_slow_down_factor = 1,
  },  
 
 {
    type = "explosion",
    name = "tl_white",
	flags = {"placeable-off-grid"},
    animation_speed = 1,
    animations =
    {
      {
        filename = "__Telelogistics__/graphics/white.png",
        priority = "extra-high",
        frame_width = 2,
        frame_height = 2,
        frame_count = 2
      }
    },
  --  light = {intensity = 0, size = 0},
 --   smoke = "smoke-fast",
 --   smoke_count = 0,
 --   smoke_slow_down_factor = 1,
  }
  
})